"""
POWER BUSINESS SYSTEMS
Copyright 2010 Power Business Systems <jasper@powerbs.co.za>
All Rights Reserved.

Licensed under The Artistic License 2.0
See LICENSE for details or
http://www.perlfoundation.org/artistic_license_2_0
"""


import os, sys
import random
import time
import datetime
from threading import Thread
from log import log
__version__ = "0.07"

class clan_lib:
    def create_rpc_str (self, type, method, args) :
        req_str = "_c:%s:%s_t:%s:%s" % (len(method), method, len(type), type) 
        if not args.get("_rfd") :
            rfd = time.mktime(datetime.datetime.now().timetuple()) 
            rfd = str(rfd) + "_" + str(random.random())
            req_str = "%s%s:%s:%s" % (req_str, "_rfd", len(rfd), rfd)
        for key in args :
            req_str = "%s%s:%s:%s" % (req_str, key, len(str(args[key])), args[key]) 
        return req_str           

    def create_rpc_resultstr (self, rarr, rfd = "") :
        res_str = "_t:9:rpc_reply" 
        if rfd != "" :
          res_str = "%s_rfd:%s:%s\n" % (res_str, len(rfd), rfd)
        if not isinstance(rarr, list) :
            return res_str    
        for row in rarr :
            for key in row :
                res_str = "%s%s:%s:%s" % (res_str, key, len(str(row[key])), row[key]) 
            res_str = res_str + "\n"
        return res_str

    def create_rpc_connerr (self, msg, rfd) :
        res = []
        res.append( {"_ERROR":str(msg)} )

        return self.create_rpc_resultstr(res, rfd)
 
    def escape_rpc_str (self, rpc_str) :
        rpc_str = str(rpc_str).replace("\r", "\\r")
        rpc_str = rpc_str.replace("\n", "\\n")
        rpc_str = rpc_str.replace("\0", "\\0")
        rpc_str = rpc_str.replace("=", ".eq.")
        rpc_str = rpc_str.replace("|", ".pipe.")
        return rpc_str


class rpc_validator:
    parent = None
    route_array = []

    def __init__(self) : 
        pass # for now

    def validate(self, mod, func, data):
        route_list = data.get("_rl")
        if not route_list :
            return False
        self.parse_route(route_list)
        ra = self.route_array
        if len(ra) == 0:
            return False
        if ra[0] == self.parent.hostname: # assume this is localhost to localhost
            return True 
        pm = self.parent.permissions
        # this still needs some work, needs to be more solid, test entire route.
        if pm.has_key(ra[0]):
            if pm[ra[0]].has_key(mod):
                if pm[ra[0]][mod].has_key(func):
                    return True
        return False

    def parse_route(self, str_list):
        self.route_array = []
        for part in str_list.split("+"):
            for h in part.split(">"):
                if h == "" : continue
                self.route_array.append(h)

class rpcClan (rpc_validator) :

    def __init__(self) :
        #rpc_validator.__init__(self)
        pass

    def buildRpcFault (self, code, text) :
        # not using the code yet..
        result = []
        result.append( {"_ERROR":text} )
        return result

    """
    def buildRpcResponse (self, args) :
        resp = ""
        i = 0
        for row in args :
            i = i + 1
            resp += ""
            for key, value in row.iteritems(): 
                resp += "" 
            resp += ""
        resp = resp + ""    
        return resp
    """ 

    def runRPC (self, method, data) :
        method = method.replace(" ", "")
        (mod, func) = method.split("::")
        if func[:1] == "_" :
            return self.buildRpcFault("4", "%s.%s is a private method" % (mod, func))
        if not os.path.isfile("modules/" + mod + ".py") and not os.path.isfile("modules/" + mod + ".pyc") :
            return self.buildRpcFault("1", "Module does not exist")
        if not self.validate(mod, func, data):
            return self.buildRpcFault("1", "Permission denied")
        try : 
            if sys.modules.get(mod) :
                sys.modules.pop(mod)
            mod_obj = __import__(mod)
            m = eval("mod_obj." + mod + "()")
            try :
                m.parent = self
                m.clan = self.clan
            except :
                pass				 
            res_args = eval("m." + func + "(data)")
        except :
            return self.buildRpcFault("3", "Error calling %s.%s: %s" % (mod, func, sys.exc_value))
        return res_args
      
       
class bridge_result :
    d_res = {}

    def __init__(self, raw_text) :
        if raw_text :
            try:
                self.parse_raw(raw_text)
            except:
                log("Clanlib: Could not parse request: %s" % raw_text)

    def get(self, name) :
        if len(self.d_res) > 1 :
            for idx in self.d_res :
                if self.d_res[idx].get(name) :
                    return self.d_res[idx].get(name)
        return ""

    def get_row(self, row_num) :
        if len(self.d_res) >= row_num :
            return self.d_res[row_num]
        return False   

    def get_all(self) :
        return self.d_res

    def get_iter(self) :
        for key in self.d_res:
            if key == 0: # skip headers
                continue
            if len(self.d_res[key]) == 0:
                continue
            yield self.d_res[key]

 
    def parse_raw(self, raw_text) :
        self.d_res = {}
        val_len = 0
        val_text = ""
        val_name = ""
        mark_start = 0
        mark_end = 0
        
        idx = 0
        row = 0
        self.d_res[row] = {}
        while 1:
            if idx >= len(raw_text) :
                break
            if raw_text[idx] == ":" :
                if mark_start > 0 :
                    mark_end = idx
                else :
                    mark_start = idx
            else : 
                if raw_text[idx] == "\n" :
                    row+= 1
                    idx+= 1
                    self.d_res[row] = {}
                    continue    
            if mark_start > 0 and mark_end > 0 :
                val_len = int(raw_text[mark_start + 1:mark_end])
                val_text = raw_text[mark_end + 1:mark_end + 1 + val_len]
                self.d_res[row][val_name] = val_text
                idx = mark_end + val_len + 1
                val_name = ""
                mark_start = 0
                mark_end = 0
            else :
                if mark_start == 0 and mark_end == 0 :
                    val_name+= raw_text[idx]
                idx+= 1
